SystemOrganization addCategory: #TusLibros!

TestCase subclass: #ShoppingCartTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCartTest commentStamp: '<historical>' prior: 0!
A ShoppingCartTest is a test class for testing the behavior of ShoppingCart!

!ShoppingCartTest methodsFor: '2-Publisher' stamp: 'JuanEdi 5/29/2013 19:54'!
testCartForCustomerShouldBeEmptyIfNoExistingOne
self assert: true.
"|aCustomer aPublisher aCart|
	aCustomer := 1.
	aPublisher := nil.
	aCart := aPublisher cartFor: aCustomer.
	self assert: aCart isEmpty"! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 5/27/2013 21:38'!
testCartShouldBeEmptyAfterCreation

	self assert: (ShoppingCart newWithCatalog: #('123')) isEmpty.! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 5/29/2013 19:56'!
testShouldContainBookAfterAddingItToACart
	| cart |
	cart := ShoppingCart newWithCatalog: #('123').
	cart add: '123' withOcurrencies: 2.
	self assert: (cart occurrencesOf: '123') = 2.
	! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 5/29/2013 20:07'!
testShouldOnlyAcceptExistingBooks
	| cart |
	cart := ShoppingCart newWithCatalog: #('123').
	self should: [ cart add: 'InexistentBook' withOcurrencies: 2 ] 
			raise: Error 
			withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart outOfCatalogMessage ].
	self assert: cart isEmpty.
	! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 5/29/2013 19:57'!
testShouldUpdateBookOrderCountAfterAddingMoreCopies
	| cart |
	cart := ShoppingCart newWithCatalog: #('123').
	cart add: '123' withOcurrencies: 2 .
	cart add: '123' withOcurrencies: 3.
	self assert: (cart occurrencesOf: '123') = 5.! !

Object subclass: #ShoppingCart
	instanceVariableNames: 'catalog products'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCart class methodsFor: 'instance-creation' stamp: 'JuanEdi 5/27/2013 21:30'!
newWithCatalog: aBookCatalog 
	
	^ self new initializeWithCatalog: aBookCatalog.! !

!ShoppingCart class methodsFor: 'exception messages' stamp: 'JuanEdi 5/29/2013 20:07'!
outOfCatalogMessage
	^ 'Error adding product to cart, the product is not in the catalog.'! !

!ShoppingCart methodsFor: 'adding' stamp: 'JuanEdi 5/29/2013 19:53'!
add: aProduct withOcurrencies: aNumber 
	| currentOcurrences |
	
	self validateCatalogContainsProduct: aProduct.
		
	currentOcurrences := products at: aProduct ifAbsent: 0.
	products at: aProduct put: (currentOcurrences + aNumber).! !

!ShoppingCart methodsFor: 'initialize-release' stamp: 'JuanEdi 5/29/2013 19:49'!
initializeWithCatalog: aCatalog 

	catalog := aCatalog.
	products := Dictionary new.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 5/29/2013 19:50'!
isEmpty

	^ products isEmpty.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 5/29/2013 19:53'!
occurrencesOf: aProduct 
	
	^ products at: aProduct.! !

!ShoppingCart methodsFor: 'validations' stamp: 'JuanEdi 5/29/2013 20:07'!
validateCatalogContainsProduct: aProduct

	(catalog includes: aProduct) ifFalse: [ self error: ShoppingCart outOfCatalogMessage]! !
